function tabRef (destination) {
    if (destination.startsWith("#")) {
        return "c" + destination.substr(1);
    } else {
        return "u" + destination;
    }
};

function IrcClient(sockJs, nick, userName, realName) {
    this.sockJs = sockJs;
    this.nick = nick;
    this.userName = userName;
    this.realName = realName;

    this.sendMsg = function (msg) {
        console.log("OUT-MSG " + msg);
        this.sockJs.send(msg);
    };

    this.sendNickMsg = function () {
        this.sendMsg("NICK " + this.nick);
    };

    this.sendUserMsg = function () {
        this.sendMsg("USER " + this.userName + " 0 * :" + this.realName);
    };

    this.sendPrivMsg = function (msg) {
        this.sendDestPrivMsg(this.selectedDestination(), msg);
    };

    this.sendDestPrivMsg = function (dest, msg) {
        this.sendMsg("PRIVMSG " + dest + " " + msg);
    };

    this.init = function () {
        this.sendNickMsg();
        this.sendUserMsg();
    };

    var ircClientRef = this;

    this.sockJs.onopen = function () {
        console.log("Socket opened");
        ircClientRef.init();
    };

    this.sockJs.onclose = function () {
        console.log("Socket closed");
    };

    this.sendPongMsg = function (nr) {
        this.sendMsg("PONG :" + nr);
    };

    this.handlePongMessage = function (payload, data) {
        console.log("PING message: " + data);
        this.sendPongMsg(data);
    };

    this.createTabEl = function (destination) {
        var tabs = $("#tabs");
        var liEl = $("<li/>");
        var aEl = $("<a/>");

        aEl.attr("href", "#" + tabRef(destination));
        aEl.attr("data-toggle", "tab");
        aEl.html(destination);


        liEl.attr("id", "label-" + tabRef(destination));
        liEl.append(aEl);
        tabs.append(liEl);
    };

    this.createTabContentEl = function (destination) {
        var tabsContent = $("#tabs-content");
        var divEl = $("<div />");
        divEl.addClass("tab-pane fade in");
        divEl.attr("id", tabRef(destination));
        tabRef(destination);

        tabsContent.append(divEl);
    };

    this.checkForDestElseCreate = function (destination) {
        var tabIdx = tabRef(destination);
        if ($("#label-" + tabIdx).length > 0) {
            return;
        }

        ircClientRef.createTabEl(destination);
        ircClientRef.createTabContentEl(destination);
    };

    this.displayDestinationMessage = function (destination, msg) {
        this.checkForDestElseCreate(destination);
        var server = $("#" + tabRef(destination));
        var logRow = $("<div/>");
        logRow.text(msg);
        server.append(logRow);
        return server;
    };

    this.selectedDestination = function () {
        return $("#tabs").find("li.active").text();
    };

    this.fetchDestination = function (data) {
        var preDestIdx = data.indexOf(" ");
        if (preDestIdx < 0) {
            return null;
        }

        var destIdx = preDestIdx + 1;
        var preDataIdx = data.indexOf(" ", destIdx);
        return data.substr(destIdx, preDataIdx - destIdx);
    };

    this.sockJs.onmessage = function (e) {
        var msg = e.data;
        if (msg.startsWith("PING ")) {
            ircClientRef.handlePongMessage(msg, msg.substr(7));
            return;
        }

        console.log("Received message: " + msg);
        var destination = ircClientRef.fetchDestination(e.data);
        var tabName = destination;
        console.log("Destination of the message: " + destination);

        var server = ircClientRef.displayDestinationMessage(tabName, msg);
    };

};